@@ -36,13 +36,13 @@ namespace tf
 # undef TF_MESSAGEFILTER_DEBUG
 #endif
 #define TF_MESSAGEFILTER_DEBUG(fmt, ...) \
-  ROS_DEBUG_NAMED("message_filter", "MessageFilter [target=%s]: "fmt, getTargetFramesString().c_str(), __VA_ARGS__)
+  ROS_DEBUG_NAMED("message_filter", "MessageFilter [target=%s]: " fmt, getTargetFramesString().c_str(), __VA_ARGS__)
 
 #ifdef TF_MESSAGEFILTER_WARN
 # undef TF_MESSAGEFILTER_WARN
 #endif
 #define TF_MESSAGEFILTER_WARN(fmt, ...) \
-  ROS_WARN_NAMED("message_filter", "MessageFilter [target=%s]: "fmt, getTargetFramesString().c_str(), __VA_ARGS__)
+  ROS_WARN_NAMED("message_filter", "MessageFilter [target=%s]: " fmt, getTargetFramesString().c_str(), __VA_ARGS__)
 
     class MessageFilterJointState : public MessageFilter<sensor_msgs::JointState>
     {
@@ -127,8 +127,8 @@ public:
 		clear();
 
 		TF_MESSAGEFILTER_DEBUG("Successful Transforms: %llu, Failed Transforms: %llu, Discarded due to age: %llu, Transform messages received: %llu, Messages received: %llu, Total dropped: %llu",
-				       (long long unsigned int)successful_transform_count_, (long long unsigned int)failed_transform_count_, 
-				       (long long unsigned int)failed_out_the_back_count_, (long long unsigned int)transform_message_count_, 
+				       (long long unsigned int)successful_transform_count_, (long long unsigned int)failed_transform_count_,
+				       (long long unsigned int)failed_out_the_back_count_, (long long unsigned int)transform_message_count_,
 				       (long long unsigned int)incoming_message_count_, (long long unsigned int)dropped_message_count_);
 
 	    }
@@ -544,12 +544,33 @@ public:
 
   virtual void onInitialize()
     {
-      tf_filter_ = new tf::MessageFilterJointState( *context_->getTFClient(),
+    	// TODO(wjwwood): remove this and use tf2 interface instead
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
+		  auto tf_client = context_->getTFClient();
+
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
+      tf_filter_ = new tf::MessageFilterJointState( *tf_client,
                                                     fixed_frame_.toStdString(), 10, update_nh_ );
 
       tf_filter_->connectInput( sub_ );
       tf_filter_->registerCallback( boost::bind( &MessageFilterJointStateDisplay::incomingMessage, this, _1 ));
+     	// TODO(wjwwood): remove this and use tf2 interface instead
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
       context_->getFrameManager()->registerFilterForTransformStatusCheck( tf_filter_, this );
+
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
     }
 
   virtual ~MessageFilterJointStateDisplay()
@@ -692,6 +713,7 @@ namespace rviz
 	void updateColorAndAlpha();
         void updateHistoryLength();
         void updateRobotDescription();
+        void updateTfPrefix();
 
         JointInfo* getJointInfo( const std::string& joint);
         JointInfo* createJoint(const std::string &joint);
@@ -726,6 +748,7 @@ namespace rviz
 	rviz::IntProperty *history_length_property_;
 
         rviz::StringProperty *robot_description_property_;
+        rviz::StringProperty *tf_prefix_property_;
         rviz::Property *joints_category_;
         rviz::BoolProperty *all_enabled_property_;
     };
