@@ -48,15 +48,8 @@
 
 #include "rviz/ogre_helpers/line.h"
 
-#include "rviz/default_plugin/markers/shape_marker.h"
-#include "rviz/default_plugin/markers/arrow_marker.h"
-#include "rviz/default_plugin/markers/line_list_marker.h"
-#include "rviz/default_plugin/markers/line_strip_marker.h"
+#include "rviz/default_plugin/marker_utils.h"
 #include "rviz/default_plugin/markers/points_marker.h"
-#include "rviz/default_plugin/markers/text_view_facing_marker.h"
-#include "rviz/default_plugin/markers/mesh_resource_marker.h"
-#include "rviz/default_plugin/markers/triangle_list_marker.h"
-#include "rviz/default_plugin/markers/marker_base.h"
 
 #include "rviz/default_plugin/interactive_markers/interactive_marker_control.h"
 #include "rviz/default_plugin/interactive_markers/interactive_marker.h"
@@ -94,64 +87,15 @@ void InteractiveMarkerControl::makeMarkers( const visualization_msgs::Interactiv
 {
   for (unsigned i = 0; i < message.markers.size(); i++)
   {
-    MarkerBasePtr marker;
-
     // create a marker with the given type
-    switch (message.markers[i].type)
-    {
-      case visualization_msgs::Marker::CUBE:
-      case visualization_msgs::Marker::CYLINDER:
-      case visualization_msgs::Marker::SPHERE:
-      {
-        marker.reset(new ShapeMarker(0, context_, markers_node_));
-      }
-        break;
+    MarkerBasePtr marker(createMarker(message.markers[i].type, 0, context_, markers_node_));
+    if (!marker) {
+      ROS_ERROR( "Unknown marker type: %d", message.markers[i].type );
+    }
 
-      case visualization_msgs::Marker::ARROW:
-      {
-        marker.reset(new ArrowMarker(0, context_, markers_node_));
-      }
-        break;
-
-      case visualization_msgs::Marker::LINE_STRIP:
-      {
-        marker.reset(new LineStripMarker(0, context_, markers_node_));
-      }
-        break;
-      case visualization_msgs::Marker::LINE_LIST:
-      {
-        marker.reset(new LineListMarker(0, context_, markers_node_));
-      }
-        break;
-      case visualization_msgs::Marker::SPHERE_LIST:
-      case visualization_msgs::Marker::CUBE_LIST:
-      case visualization_msgs::Marker::POINTS:
-      {
-        PointsMarkerPtr points_marker;
-        points_marker.reset(new PointsMarker(0, context_, markers_node_));
-        points_markers_.push_back( points_marker );
-        marker = points_marker;
-      }
-        break;
-      case visualization_msgs::Marker::TEXT_VIEW_FACING:
-      {
-        marker.reset(new TextViewFacingMarker(0, context_, markers_node_));
-      }
-        break;
-      case visualization_msgs::Marker::MESH_RESOURCE:
-      {
-        marker.reset(new MeshResourceMarker(0, context_, markers_node_));
-      }
-        break;
-
-      case visualization_msgs::Marker::TRIANGLE_LIST:
-      {
-        marker.reset(new TriangleListMarker(0, context_, markers_node_));
-      }
-        break;
-      default:
-        ROS_ERROR( "Unknown marker type: %d", message.markers[i].type );
-        break;
+    PointsMarkerPtr points_marker = boost::dynamic_pointer_cast<PointsMarker>(marker);
+    if (points_marker) {
+      points_markers_.push_back( points_marker );
     }
 
     visualization_msgs::MarkerPtr marker_msg( new visualization_msgs::Marker(message.markers[ i ]) );
@@ -619,7 +563,7 @@ void InteractiveMarkerControl::moveViewPlane( Ogre::Ray &mouse_ray, const Viewpo
 {
   // find plane on which mouse is moving
   Ogre::Plane plane( event.viewport->getCamera()->getRealDirection(),
-                     parent_position_at_mouse_down_ + parent_to_grab_position_);
+                     grab_point_in_reference_frame_);
 
   // find intersection of mouse with the plane
   std::pair<bool, Ogre::Real> intersection = mouse_ray.intersects(plane);
@@ -628,7 +572,7 @@ void InteractiveMarkerControl::moveViewPlane( Ogre::Ray &mouse_ray, const Viewpo
   Ogre::Vector3 mouse_position_on_plane = mouse_ray.getPoint(intersection.second);
 
   // move parent so grab position relative to parent coincides with new mouse position.
-  parent_->setPose( mouse_position_on_plane - parent_to_grab_position_,
+  parent_->setPose( mouse_position_on_plane - grab_point_in_reference_frame_ + parent_position_at_mouse_down_,
                     parent_->getOrientation(),
                     name_ );
 }
@@ -1302,14 +1246,12 @@ void InteractiveMarkerControl::beginMouseMovement( ViewportMouseEvent& event, bo
     // If we couldn't get a 3D point for the grab, just use the
     // current relative position of the control frame.
     grab_point_in_reference_frame_ = control_frame_node_->getPosition();
-    parent_to_grab_position_ = Ogre::Vector3(0,0,0);
   }
   else
   {
     // If we could get a 3D point for the grab, convert it from
     // the world frame to the reference frame (in case those are different).
     grab_point_in_reference_frame_ = reference_node_->convertWorldToLocalPosition(grab_point_in_world_frame);
-    parent_to_grab_position_ = grab_point_in_world_frame - parent_->getPosition();
   }
   parent_position_at_mouse_down_ = parent_->getPosition();
   parent_orientation_at_mouse_down_ = parent_->getOrientation();
