@@ -206,6 +206,8 @@ TFDisplay::~TFDisplay()
 
 void TFDisplay::onInitialize()
 {
+  frame_config_enabled_state_.clear();
+
   root_node_ = scene_node_->createChildSceneNode();
 
   names_node_ = root_node_->createChildSceneNode();
@@ -213,6 +215,27 @@ void TFDisplay::onInitialize()
   axes_node_ = root_node_->createChildSceneNode();
 }
 
+void TFDisplay::load(const Config& config)
+{
+  Display::load(config);
+
+  // Load the enabled state for all frames specified in the config, and store
+  // the values in a map so that the enabled state can be properly set once
+  // the frame is created
+  Config c = config.mapGetChild("Frames");
+  for( Config::MapIterator iter = c.mapIterator(); iter.isValid(); iter.advance() )
+  {
+    QString key = iter.currentKey();
+    if( key != "All Enabled" )
+    {
+      const Config& child = iter.currentChild();
+      bool enabled = child.mapGetChild("Value").getValue().toBool();
+
+      frame_config_enabled_state_[key.toStdString()] = enabled;
+    }
+  }
+}
+
 void TFDisplay::clear()
 {
   // Clear the tree.
@@ -345,7 +368,17 @@ void TFDisplay::updateFrames()
 {
   typedef std::vector<std::string> V_string;
   V_string frames;
+  // TODO(wjwwood): remove this and use tf2 interface instead
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
   context_->getTFClient()->getFrameStrings( frames );
+
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
   std::sort(frames.begin(), frames.end());
 
   S_FrameInfo current_frames;
@@ -414,7 +447,7 @@ FrameInfo* TFDisplay::createFrame(const std::string& frame)
   info->selection_handler_.reset( new FrameSelectionHandler( info, this, context_ ));
   info->selection_handler_->addTrackedObjects( info->axes_->getSceneNode() );
 
-  info->name_text_ = new MovableText( frame, "Arial", 0.1 );
+  info->name_text_ = new MovableText( frame, "Liberation Sans", 0.1 );
   info->name_text_->setTextAlignment(MovableText::H_CENTER, MovableText::V_BELOW);
   info->name_node_ = names_node_->createChildSceneNode();
   info->name_node_->attachObject( info->name_text_ );
@@ -452,6 +485,13 @@ FrameInfo* TFDisplay::createFrame(const std::string& frame)
                                                         info->enabled_property_ );
   info->rel_orientation_property_->setReadOnly( true );
 
+  // If the current frame was specified as disabled in the config file
+  // then its enabled state must be updated accordingly
+  if( frame_config_enabled_state_.count(frame) > 0 && !frame_config_enabled_state_[frame] )
+  {
+      info->enabled_property_->setBool(false);
+  }
+
   updateFrame( info );
 
   return info;
@@ -464,8 +504,18 @@ Ogre::ColourValue lerpColor(const Ogre::ColourValue& start, const Ogre::ColourVa
 
 void TFDisplay::updateFrame( FrameInfo* frame )
 {
+  // TODO(wjwwood): remove this and use tf2 interface instead
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
   tf::TransformListener* tf = context_->getTFClient();
 
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
+
   // Check last received time so we can grey out/fade out frames that have stopped being published
   ros::Time latest_time;
   tf->getLatestCommonTime( fixed_frame_.toStdString(), frame->name_, latest_time, 0 );
@@ -587,7 +637,18 @@ void TFDisplay::updateFrame( FrameInfo* frame )
 
     tf::StampedTransform transform;
     try {
-      context_->getFrameManager()->getTFClientPtr()->lookupTransform(frame->parent_,frame->name_,ros::Time(0),transform);
+      // TODO(wjwwood): remove this and use tf2 interface instead
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
+      auto tf_client = context_->getFrameManager()->getTFClientPtr();
+
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
+      tf_client->lookupTransform(frame->parent_,frame->name_,ros::Time(0),transform);
     }
     catch(tf::TransformException& e)
     {
@@ -751,10 +812,13 @@ void FrameInfo::setEnabled( bool enabled )
     display_->changing_single_frame_enabled_state_ = false;
   }
 
+  // Update the configuration that stores the enabled state of all frames
+  display_->frame_config_enabled_state_[this->name_] = enabled;
+
   display_->context_->queueRender();
 }
 
 } // namespace rviz
 
-#include <pluginlib/class_list_macros.h>
+#include <pluginlib/class_list_macros.hpp>
 PLUGINLIB_EXPORT_CLASS( rviz::TFDisplay, rviz::Display )
