@@ -48,8 +48,7 @@
 #include <OgreSharedPtr.h>
 #include <OgreTechnique.h>
 
-#include <tinyxml.h>
-
+#include <tinyxml2.h>
 
 #include <ros/assert.h>
 
@@ -577,7 +576,7 @@ float getMeshUnitRescale(const std::string& resource_path)
    
 
   // Try to read unit to meter conversion ratio from mesh. Only valid in Collada XML formats. 
-  TiXmlDocument xmlDoc;
+  tinyxml2::XMLDocument xmlDoc;
   float unit_scale(1.0);
   resource_retriever::Retriever retriever;
   resource_retriever::MemoryResource res;
@@ -604,19 +603,19 @@ float getMeshUnitRescale(const std::string& resource_path)
   // Find the appropriate element if it exists
   if(!xmlDoc.Error())
   {
-    TiXmlElement * colladaXml = xmlDoc.FirstChildElement("COLLADA");
+    tinyxml2::XMLElement * colladaXml = xmlDoc.FirstChildElement("COLLADA");
     if(colladaXml)
     {
-      TiXmlElement *assetXml = colladaXml->FirstChildElement("asset");
+      tinyxml2::XMLElement *assetXml = colladaXml->FirstChildElement("asset");
       if(assetXml)
       {
-        TiXmlElement *unitXml = assetXml->FirstChildElement("unit");
+        tinyxml2::XMLElement *unitXml = assetXml->FirstChildElement("unit");
         if (unitXml && unitXml->Attribute("meter"))
         {
           // Failing to convert leaves unit_scale as the default.
           if(unitXml->QueryFloatAttribute("meter", &unit_scale) != 0)
             ROS_WARN_STREAM("getMeshUnitRescale::Failed to convert unit element meter attribute to determine scaling. unit element: "
-                            << *unitXml);
+                            << unitXml->GetText());
         }
       }
     }
