@@ -30,6 +30,7 @@
 #include <fstream>
 
 #include <QAction>
+#include <QShortcut>
 #include <QApplication>
 #include <QCloseEvent>
 #include <QDesktopServices>
@@ -47,6 +48,7 @@
 #include <QLabel>
 #include <QToolButton>
 #include <QHBoxLayout>
+#include <QTabBar>
 
 #include <boost/algorithm/string/split.hpp>
 #include <boost/algorithm/string/trim.hpp>
@@ -68,6 +70,8 @@
 #include "rviz/help_panel.h"
 #include "rviz/loading_dialog.h"
 #include "rviz/new_object_dialog.h"
+#include "rviz/preferences.h"
+#include "rviz/preferences_dialog.h"
 #include "rviz/panel_dock_widget.h"
 #include "rviz/panel_factory.h"
 #include "rviz/render_panel.h"
@@ -91,7 +95,7 @@
 namespace fs = boost::filesystem;
 
 #define CONFIG_EXTENSION "rviz"
-#define CONFIG_EXTENSION_WILDCARD "*."CONFIG_EXTENSION
+#define CONFIG_EXTENSION_WILDCARD "*." CONFIG_EXTENSION
 #define RECENT_CONFIG_COUNT 10
 
 #if BOOST_FILESYSTEM_VERSION == 3
@@ -107,6 +111,7 @@ namespace rviz
 
 VisualizationFrame::VisualizationFrame( QWidget* parent )
   : QMainWindow( parent )
+  , app_(NULL)
   , render_panel_(NULL)
   , show_help_action_(NULL)
   , file_menu_(NULL)
@@ -123,6 +128,7 @@ VisualizationFrame::VisualizationFrame( QWidget* parent )
   , loading_( false )
   , post_load_timer_( new QTimer( this ))
   , frame_count_(0)
+  , preferences_( new Preferences() )
 {
   panel_factory_ = new PanelFactory();
 
@@ -168,6 +174,11 @@ VisualizationFrame::~VisualizationFrame()
   delete panel_factory_;
 }
 
+void VisualizationFrame::setApp( QApplication * app )
+{
+  app_ = app;
+}
+
 void VisualizationFrame::setStatus( const QString & message )
 {
   Q_EMIT statusUpdate( message );
@@ -255,12 +266,19 @@ void VisualizationFrame::initialize(const QString& display_config_file )
   }
   Q_EMIT statusUpdate( "Initializing" );
 
+  // Periodically process events for the splash screen.
+  // See: http://doc.qt.io/qt-5/qsplashscreen.html#details
+  if (app_) app_->processEvents();
+
   if( !ros::isInitialized() )
   {
     int argc = 0;
     ros::init( argc, 0, "rviz", ros::init_options::AnonymousName );
   }
 
+  // Periodically process events for the splash screen.
+  if (app_) app_->processEvents();
+
   QWidget* central_widget = new QWidget(this);
   QHBoxLayout* central_layout = new QHBoxLayout;
   central_layout->setSpacing(0);
@@ -294,17 +312,35 @@ void VisualizationFrame::initialize(const QString& display_config_file )
 
   central_widget->setLayout( central_layout );
 
+  // Periodically process events for the splash screen.
+  if (app_) app_->processEvents();
+
   initMenus();
 
+  // Periodically process events for the splash screen.
+  if (app_) app_->processEvents();
+
   initToolbars();
 
+  // Periodically process events for the splash screen.
+  if (app_) app_->processEvents();
+
   setCentralWidget( central_widget );
 
+  // Periodically process events for the splash screen.
+  if (app_) app_->processEvents();
+
   manager_ = new VisualizationManager( render_panel_, this );
   manager_->setHelpPath( help_path_ );
 
+  // Periodically process events for the splash screen.
+  if (app_) app_->processEvents();
+
   render_panel_->initialize( manager_->getSceneManager(), manager_ );
 
+  // Periodically process events for the splash screen.
+  if (app_) app_->processEvents();
+
   ToolManager* tool_man = manager_->getToolManager();
 
   connect( manager_, SIGNAL( configChanged() ), this, SLOT( setDisplayConfigModified() ));
@@ -315,6 +351,9 @@ void VisualizationFrame::initialize(const QString& display_config_file )
 
   manager_->initialize();
 
+  // Periodically process events for the splash screen.
+  if (app_) app_->processEvents();
+
   if( display_config_file != "" )
   {
     loadDisplayConfig( display_config_file );
@@ -324,6 +363,9 @@ void VisualizationFrame::initialize(const QString& display_config_file )
     loadDisplayConfig( QString::fromStdString( default_display_config_file_ ));
   }
 
+  // Periodically process events for the splash screen.
+  if (app_) app_->processEvents();
+
   delete splash_;
   splash_ = 0;
 
@@ -341,7 +383,7 @@ void VisualizationFrame::initConfigs()
 
   config_dir_ = (fs::path(home_dir_) / ".rviz").BOOST_FILE_STRING();
   persistent_settings_file_ = (fs::path(config_dir_) / "persistent_settings").BOOST_FILE_STRING();
-  default_display_config_file_ = (fs::path(config_dir_) / "default."CONFIG_EXTENSION).BOOST_FILE_STRING();
+  default_display_config_file_ = (fs::path(config_dir_) / "default." CONFIG_EXTENSION).BOOST_FILE_STRING();
 
   if( fs::is_regular_file( config_dir_ ))
   {
@@ -371,7 +413,7 @@ void VisualizationFrame::loadPersistentSettings()
       last_config_dir_ = last_config_dir.toStdString();
       last_image_dir_ = last_image_dir.toStdString();
     }
-    
+
     Config recent_configs_list = config.mapGetChild( "Recent Configs" );
     recent_configs_.clear();
     int num_recent = recent_configs_list.listLength();
@@ -409,9 +451,14 @@ void VisualizationFrame::savePersistentSettings()
 void VisualizationFrame::initMenus()
 {
   file_menu_ = menuBar()->addMenu( "&File" );
-  file_menu_->addAction( "&Open Config", this, SLOT( onOpen() ), QKeySequence( "Ctrl+O" ));
-  file_menu_->addAction( "&Save Config", this, SLOT( onSave() ), QKeySequence( "Ctrl+S" ));
-  file_menu_->addAction( "Save Config &As", this, SLOT( onSaveAs() ));
+
+  QAction * file_menu_open_action = file_menu_->addAction( "&Open Config", this, SLOT( onOpen() ), QKeySequence( "Ctrl+O" ));
+  this->addAction(file_menu_open_action);
+  QAction * file_menu_save_action = file_menu_->addAction( "&Save Config", this, SLOT( onSave() ), QKeySequence( "Ctrl+S" ));
+  this->addAction(file_menu_save_action);
+  QAction * file_menu_save_as_action = file_menu_->addAction( "Save Config &As", this, SLOT( onSaveAs() ), QKeySequence( "Ctrl+Shift+S"));
+  this->addAction(file_menu_save_as_action);
+
   recent_configs_menu_ = file_menu_->addMenu( "&Recent Configs" );
   file_menu_->addAction( "Save &Image", this, SLOT( onSaveImage() ));
   if( show_choose_new_master_option_ )
@@ -420,12 +467,21 @@ void VisualizationFrame::initMenus()
     file_menu_->addAction( "Change &Master", this, SLOT( changeMaster() ));
   }
   file_menu_->addSeparator();
-  file_menu_->addAction( "&Quit", this, SLOT( close() ), QKeySequence( "Ctrl+Q" ));
+  file_menu_->addAction( "&Preferences", this, SLOT( openPreferencesDialog() ), QKeySequence( "Ctrl+P" ));
+
+  QAction * file_menu_quit_action = file_menu_->addAction( "&Quit", this, SLOT( close() ), QKeySequence( "Ctrl+Q" ));
+  this->addAction(file_menu_quit_action);
 
   view_menu_ = menuBar()->addMenu( "&Panels" );
   view_menu_->addAction( "Add &New Panel", this, SLOT( openNewPanelDialog() ));
   delete_view_menu_ = view_menu_->addMenu( "&Delete Panel" );
   delete_view_menu_->setEnabled( false );
+
+  QAction * fullscreen_action = view_menu_->addAction("&Fullscreen", this, SLOT( setFullScreen(bool) ), Qt::Key_F11);
+  fullscreen_action->setCheckable(true);
+  this->addAction(fullscreen_action); // Also add to window, or the shortcut doest work when the menu is hidden.
+  connect(this, SIGNAL( fullScreenChange( bool ) ), fullscreen_action, SLOT( setChecked( bool ) ) );
+  new QShortcut(Qt::Key_Escape, this, SLOT( exitFullScreen() ));
   view_menu_->addSeparator();
 
   QMenu* help_menu = menuBar()->addMenu( "&Help" );
@@ -532,6 +588,20 @@ void VisualizationFrame::onDockPanelVisibilityChange( bool visible )
 
 }
 
+void VisualizationFrame::openPreferencesDialog()
+{
+  Preferences temp_preferences( *preferences_.get() );
+  PreferencesDialog* dialog = new PreferencesDialog( panel_factory_,
+                                                 &temp_preferences,
+                                                 this );
+  manager_->stopUpdate();
+  if( dialog->exec() == QDialog::Accepted ) {
+    // Apply preferences.
+    preferences_ = boost::make_shared<Preferences>( temp_preferences );
+  }
+  manager_->startUpdate();
+}
+
 void VisualizationFrame::openNewPanelDialog()
 {
   QString class_id;
@@ -548,7 +618,11 @@ void VisualizationFrame::openNewPanelDialog()
   manager_->stopUpdate();
   if( dialog->exec() == QDialog::Accepted )
   {
-    addPanelByName( display_name, class_id );
+    QDockWidget *dock = addPanelByName( display_name, class_id );
+    if ( dock )
+    {
+      connect( dock, SIGNAL( dockLocationChanged( Qt::DockWidgetArea )), this, SLOT( onDockPanelChange() ) );
+    }
   }
   manager_->startUpdate();
 }
@@ -625,7 +699,7 @@ void VisualizationFrame::loadDisplayConfig( const QString& qpath )
   std::string actual_load_path = path;
   if( !fs::exists( path ) || fs::is_directory( path ) || fs::is_empty( path ))
   {
-    actual_load_path = (fs::path(package_path_) / "default.rviz").BOOST_FILE_STRING();      
+    actual_load_path = (fs::path(package_path_) / "default.rviz").BOOST_FILE_STRING();
     if( !fs::exists( actual_load_path ))
     {
       ROS_ERROR( "Default display config '%s' not found.  RViz will be very empty at first.", actual_load_path.c_str() );
@@ -684,7 +758,10 @@ void VisualizationFrame::setDisplayConfigModified()
 {
   if( !loading_ )
   {
-    setWindowModified( true );
+    if( !isWindowModified() )
+    {
+      setWindowModified( true );
+    }
   }
 }
 
@@ -731,6 +808,7 @@ void VisualizationFrame::save( Config config )
   manager_->save( config.mapMakeChild( "Visualization Manager" ));
   savePanels( config.mapMakeChild( "Panels" ));
   saveWindowGeometry( config.mapMakeChild( "Window Geometry" ));
+  savePreferences( config.mapMakeChild( "Preferences" ));
 }
 
 void VisualizationFrame::load( const Config& config )
@@ -738,6 +816,7 @@ void VisualizationFrame::load( const Config& config )
   manager_->load( config.mapGetChild( "Visualization Manager" ));
   loadPanels( config.mapGetChild( "Panels" ));
   loadWindowGeometry( config.mapGetChild( "Window Geometry" ));
+  loadPreferences( config.mapGetChild( "Preferences" ));
 }
 
 void VisualizationFrame::loadWindowGeometry( const Config& config )
@@ -754,7 +833,7 @@ void VisualizationFrame::loadWindowGeometry( const Config& config )
       config.mapGetInt( "Height", &height ))
   {
     resize( width, height );
-  }    
+  }
 
   QString main_window_config;
   if( config.mapGetString( "QMainWindow State", &main_window_config ))
@@ -832,6 +911,7 @@ void VisualizationFrame::loadPanels( const Config& config )
       // qobject_cast.
       if( dock )
       {
+        connect(dock, SIGNAL( dockLocationChanged( Qt::DockWidgetArea )), this, SLOT( onDockPanelChange() ) );
         Panel* panel = qobject_cast<Panel*>( dock->widget() );
         if( panel )
         {
@@ -840,6 +920,8 @@ void VisualizationFrame::loadPanels( const Config& config )
       }
     }
   }
+
+  onDockPanelChange();
 }
 
 void VisualizationFrame::savePanels( Config config )
@@ -852,6 +934,16 @@ void VisualizationFrame::savePanels( Config config )
   }
 }
 
+void VisualizationFrame::loadPreferences( const Config& config )
+{
+  config.mapGetBool( "PromptSaveOnExit", &(preferences_->prompt_save_on_exit) );
+}
+
+void VisualizationFrame::savePreferences( Config config )
+{
+  config.mapSetValue( "PromptSaveOnExit", preferences_->prompt_save_on_exit );
+}
+
 bool VisualizationFrame::prepareToExit()
 {
   if( !initialized_ )
@@ -861,7 +953,7 @@ bool VisualizationFrame::prepareToExit()
 
   savePersistentSettings();
 
-  if( isWindowModified() )
+  if( isWindowModified() && preferences_->prompt_save_on_exit )
   {
     QMessageBox box( this );
     box.setText( "There are unsaved changes." );
@@ -897,7 +989,7 @@ bool VisualizationFrame::prepareToExit()
         default:
           return false;
         }
-        
+
       }
     case QMessageBox::Discard:
       return true;
@@ -972,9 +1064,9 @@ void VisualizationFrame::onSaveAs()
   {
     std::string filename = q_filename.toStdString();
     fs::path path( filename );
-    if( path.extension() != "."CONFIG_EXTENSION )
+    if( path.extension() != "." CONFIG_EXTENSION )
     {
-      filename += "."CONFIG_EXTENSION;
+      filename += "." CONFIG_EXTENSION;
     }
 
     if( !saveDisplayConfig( QString::fromStdString( filename )))
@@ -1125,10 +1217,13 @@ void VisualizationFrame::onHelpAbout()
   QString about_text = QString(
     "This is RViz version %1 (%2).\n"
     "\n"
-    "Compiled against OGRE version %3.%4.%5%6 (%7)."
+    "Compiled against Qt version %3."
+    "\n"
+    "Compiled against OGRE version %4.%5.%6%7 (%8)."
   )
   .arg(get_version().c_str())
   .arg(get_distro().c_str())
+  .arg(QT_VERSION_STR)
   .arg(OGRE_VERSION_MAJOR)
   .arg(OGRE_VERSION_MINOR)
   .arg(OGRE_VERSION_PATCH)
@@ -1138,6 +1233,15 @@ void VisualizationFrame::onHelpAbout()
   QMessageBox::about(QApplication::activeWindow(), "About", about_text);
 }
 
+void VisualizationFrame::onDockPanelChange()
+{
+  QList<QTabBar *> tab_bars = findChildren<QTabBar *>(QString(), Qt::FindDirectChildrenOnly);
+  for ( QList<QTabBar *>::iterator it = tab_bars.begin(); it != tab_bars.end(); it++ )
+  {
+    (*it)->setElideMode( Qt::ElideNone );
+  }
+}
+
 QWidget* VisualizationFrame::getParentWindow()
 {
   return this;
@@ -1171,6 +1275,29 @@ void VisualizationFrame::onDeletePanel()
   }
 }
 
+void VisualizationFrame::setFullScreen( bool full_screen )
+{
+  Q_EMIT( fullScreenChange( full_screen ) );
+
+  if (full_screen)
+    toolbar_visible_ = toolbar_->isVisible();
+  menuBar()->setVisible(!full_screen);
+  toolbar_->setVisible(!full_screen && toolbar_visible_);
+  statusBar()->setVisible(!full_screen);
+  setHideButtonVisibility(!full_screen);
+
+  if (full_screen)
+    setWindowState(windowState() | Qt::WindowFullScreen);
+  else
+    setWindowState(windowState() & ~Qt::WindowFullScreen);
+  show();
+}
+
+void VisualizationFrame::exitFullScreen()
+{
+  setFullScreen( false );
+}
+
 QDockWidget* VisualizationFrame::addPanelByName( const QString& name,
                                                  const QString& class_id,
                                                  Qt::DockWidgetArea area,
@@ -1211,6 +1338,7 @@ PanelDockWidget* VisualizationFrame::addPane( const QString& name, QWidget* pane
 
   // we want to know when that panel becomes visible
   connect( dock, SIGNAL( visibilityChanged( bool )), this, SLOT( onDockPanelVisibilityChange( bool ) ));
+  connect( this, SIGNAL( fullScreenChange(bool) ), dock, SLOT( overrideVisibility(bool) ));
 
   QAction* toggle_action = dock->toggleViewAction();
   view_menu_->addAction( toggle_action );
