@@ -33,6 +33,9 @@
 #include <QCursor>
 #include <QPixmap>
 #include <QTimer>
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+#include <QWindow>
+#endif
 
 #include <boost/bind.hpp>
 
@@ -114,7 +117,21 @@ public:
   boost::mutex render_mutex_;
 };
 
-VisualizationManager::VisualizationManager( RenderPanel* render_panel, WindowManagerInterface* wm, boost::shared_ptr<tf::TransformListener> tf )
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+VisualizationManager::VisualizationManager(RenderPanel* render_panel, WindowManagerInterface * wm)
+: VisualizationManager(render_panel, wm, boost::shared_ptr<tf::TransformListener>())
+{}
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
+
+VisualizationManager::VisualizationManager(
+  RenderPanel* render_panel,
+  WindowManagerInterface* wm,
+  boost::shared_ptr<tf::TransformListener> tf)
 : ogre_root_( Ogre::Root::getSingletonPtr() )
 , update_timer_(0)
 , shutting_down_(false)
@@ -125,10 +142,21 @@ VisualizationManager::VisualizationManager( RenderPanel* render_panel, WindowMan
 , frame_count_(0)
 , window_manager_(wm)
 , private_( new VisualizationManagerPrivate )
-, default_visibility_bit_( visibility_bit_allocator_.allocBit() )
 {
+  // visibility_bit_allocator_ is listed after default_visibility_bit_ (and thus initialized later be default):
+  default_visibility_bit_ = visibility_bit_allocator_.allocBit();
+
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
   frame_manager_ = new FrameManager(tf);
 
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
+
   render_panel->setAutoRender(false);
 
   private_->threaded_nh_.setCallbackQueue(&private_->threaded_queue_);
@@ -147,7 +175,7 @@ VisualizationManager::VisualizationManager( RenderPanel* render_panel, WindowMan
   display_property_tree_model_ = new PropertyTreeModel( root_display_group_ );
   display_property_tree_model_->setDragDropClass( "display" );
   connect( display_property_tree_model_, SIGNAL( configChanged() ), this, SIGNAL( configChanged() ));
-  
+
   tool_manager_ = new ToolManager( this );
   connect( tool_manager_, SIGNAL( configChanged() ), this, SIGNAL( configChanged() ));
   connect( tool_manager_, SIGNAL( toolChanged( Tool* ) ), this, SLOT( onToolChanged( Tool* ) ));
@@ -173,6 +201,10 @@ VisualizationManager::VisualizationManager( RenderPanel* render_panel, WindowMan
                                    "RViz will try to render this many frames per second.",
                                    global_options_, SLOT( updateFps() ), this );
 
+  default_light_enabled_property_ = new BoolProperty( "Default Light", true,
+                                                      "Light source attached to the current 3D view.",
+                                                      global_options_, SLOT( updateDefaultLightVisible() ), this );
+
   root_display_group_->initialize( this ); // only initialize() a Display after its sub-properties are created.
   root_display_group_->setEnabled( true );
 
@@ -384,7 +416,14 @@ void VisualizationManager::updateFrames()
 {
   typedef std::vector<std::string> V_string;
   V_string frames;
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   frame_manager_->getTFClient()->getFrameStrings( frames );
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 
   // Check the fixed frame to see if it's ok
   std::string error;
@@ -412,13 +451,32 @@ void VisualizationManager::updateFrames()
 
 tf::TransformListener* VisualizationManager::getTFClient() const
 {
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   return frame_manager_->getTFClient();
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
+}
+
+std::shared_ptr<tf2_ros::Buffer> VisualizationManager::getTF2BufferPtr() const
+{
+  return frame_manager_->getTF2BufferPtr();
 }
 
 void VisualizationManager::resetTime()
 {
   root_display_group_->reset();
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   frame_manager_->getTFClient()->clear();
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 
   ros_time_begin_ = ros::Time();
   wall_clock_begin_ = ros::WallTime();
@@ -513,6 +571,11 @@ void VisualizationManager::updateFps()
   }
 }
 
+void VisualizationManager::updateDefaultLightVisible()
+{
+  directional_light_->setVisible(default_light_enabled_property_->getBool());
+}
+
 void VisualizationManager::handleMouseEvent( const ViewportMouseEvent& vme )
 {
   //process pending mouse events
@@ -522,6 +585,17 @@ void VisualizationManager::handleMouseEvent( const ViewportMouseEvent& vme )
   if( current_tool )
   {
     ViewportMouseEvent _vme = vme;
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+    QWindow* window = vme.panel->windowHandle();
+    if (window)
+    {
+        double pixel_ratio = window->devicePixelRatio();
+        _vme.x = static_cast<int>(pixel_ratio * _vme.x);
+        _vme.y = static_cast<int>(pixel_ratio * _vme.y);
+        _vme.last_x = static_cast<int>(pixel_ratio * _vme.last_x);
+        _vme.last_y = static_cast<int>(pixel_ratio * _vme.last_y);
+    }
+#endif
     flags = current_tool->processMouseEvent( _vme );
     vme.panel->setCursor( current_tool->getCursor() );
   }
